@@ -8,7 +8,7 @@ use warnings;
 # as x.yy_zz so it will not be listed by CPAN.pm as the last.
 # After test results will be ok, it can be officially released x.(yy+1) version.
 
-our $VERSION = '0.34';
+our $VERSION = '0.35';
 $VERSION = eval $VERSION;
 
 1;
@@ -28,7 +28,7 @@ Then point your browser to http://localhost:3000 and use the gui.
 =head1 DESCRIPTION
 
 Perl Node Interface is a node programming IDE via browser. 
-It is thinked to give Perl power to people who don-t know Perl.
+It's thought to give Perl power to people who don-t know Perl.
 It is very easy for Perl coders to extend it adding new nodes.
 
 =head1 SEE ALSO
@@ -37,24 +37,6 @@ L<PNI::Guides>
 
 L<PNI demo|http://pnidemo-fibo.dotcloud.com>
 
-=head1 DEVELOPMENT
-
-L<PNI repository|http://github.com/fibo/pni-pm>
-
-=head1 RESOURCES
-
-L<PNI blog|http://perl-node-interface.blogspot.com>
-
-L<PNI class diagram|http://goo.gl/MQ89f>
-
-L<PNI node coverage|http://goo.gl/hfAoU>
-
-=head1 SUPPORT
-
-L<#pni IRC channel on freenode|irc://freenode/pni>
-
-L<PNI mailing list|http://groups.google.com/group/perl-node-interface>
-
 =head1 AUTHOR
 
 G. Casati , E<lt>fibo@cpan.orgE<gt>
