@@ -3,40 +3,48 @@
 
 var $j = jQuery.noConflict();
 
-//require(['THREE/Scene']);
 
 // PNI //[
 
 PNI = {
 
-    // Global map id <--> object.
-    // TODO si potrebbe anche togliere, sarebbe meglio.
-    // in particolare l' edge deve essere creato da zero, e il semiEdge deve
-    // essere una classe a parte.
+    // Global map id <--> PNI.Elem instance.
     elemById: {},
 
-    extend: function(subClass, superClass){
+    // Inheritance.
+    extend: function(subClass, superClass) { //[
 
         for (var key in superClass.prototype) {
 
-            if (superClass.prototype.hasOwnProperty(key)) {
-
+            if ( 
+            // Inherit from superClass ...
+            superClass.prototype.hasOwnProperty(key) 
+            && 
+            // except when subClass overrides ...
+            ! subClass.prototype.hasOwnProperty(key) 
+            && 
+            // and when key is __SUPER__
+            // TODO TODO TODO ancora non la uso sta super
+            key != '__SUPER__'
+            ) {
+                
                 subClass.prototype[key] = superClass.prototype[key];
 
             }
         }
-    },
+
+    }, //]
 
     Theme: {
 
         fontFamily: "Calibri",
         fontSize: 12,
 
-        edgeStrokeStyle: "blue",
+        edgeStrokeStyle: "black",
         edgeLineWidth: 2,
 
         nodeBoxTransitionDuration: 0.2,
-        nodeFill: "orange",
+        nodeFill: "grey",
         nodeHeight: 30,
 
         slotSide: 10,
@@ -88,7 +96,8 @@ PNI.Scenario = function(arg) { //[
     var height = this.$container.height();
     var width = this.$container.width();
 
-    this.nodeSelector = new PNI.NodeSelector({scenario:this});
+    this.nodeSelector = new PNI.NodeSelector({scenario:this} );
+    this.toolbar = new PNI.Toolbar({scenario:this} );
     this.nodeSelector.update();
 
     this.edges = [];
@@ -97,16 +106,27 @@ PNI.Scenario = function(arg) { //[
     this.stage = new Kinetic.Stage("scenario",width,height);
     this.layer = new Kinetic.Layer();
 
-    this.currenSlot = undefined;
-    this.currenNode = undefined;
-    this.semiEdge = undefined;
+    this.currentSlot = undefined;
+    this.currentNode = undefined;
+    this.semiEdge = undefined; // TODO this.semiEdge = new PNI.SemiEdge();
 
     this.updateView();
 
+    // events //[
+
     var that = this;
 
-    this.$container
-    .dblclick( function(evt) {
+    //this.stage.on( 'click', function(evt) { console.log(evt); } );
+    //this.stage.on( 'keypress', function(evt) { console.log(evt); } );
+
+    this.$container //[
+
+    // In teoria dovrei ascoltare gli eventi del Canvas e non del suo div
+    // contenitore, altrimenti mi arriva prima l' evento dal canv as e poi
+    // quello da jQuery.
+    //.click(function(evt) {console.log(evt);} )
+
+    .dblclick( function(evt) { //[
 
         var nodeSelector = that.nodeSelector;
 
@@ -135,22 +155,32 @@ PNI.Scenario = function(arg) { //[
                     type: ui.item.value,
                     x: x,
                     y: y
-                });
+                } );
             }
-        });
-    })
-    .keypress( function( evt ) {
+        } );
+    } ) //]
 
-        if( evt.charCode == 13 ) {
+    .keypress( function( evt ) { //[
+
+        // # char.
+        if( evt.charCode == 35 ) { //[
+
+            if ( that.nodeSelector.isVisible === true ) {
+
+                that.addNode({
+                    type:"PNI::Comment",
+                    x:that.nodeSelector.x,
+                    y:that.nodeSelector.y
+                } );
+
+                that.nodeSelector.hide();
 
-            if ( that.nodeSelector.isVisible === false ) {
-                that.runTask();
-                that.updateView();
             }
 
-        }
+        } //]
 
-        if( evt.charCode == 36 ) {
+        // $ char.
+        if( evt.charCode == 36 ) { //[
 
             if ( that.nodeSelector.isVisible === true ) {
 
@@ -161,25 +191,57 @@ PNI.Scenario = function(arg) { //[
                     type:"Perldata::Scalar",
                     x:that.nodeSelector.x,
                     y:that.nodeSelector.y
-                });
+                } );
 
                 that.nodeSelector.hide();
 
             }
 
-        }
-        if( evt.charCode == 37 ) {
-            console.log("TODO: create hash (dove si trova il mouse)");
-        }
-        if( evt.charCode == 64 ) {
-            console.log("TODO: create array (dove si trova il mouse)");
-        }
-    });
+        } //]
+
+        // % char.
+        if( evt.charCode == 37 ) { //[
+
+            if ( that.nodeSelector.isVisible === true ) {
+
+                that.addNode({
+                    type:"Perldata::Hash",
+                    x:that.nodeSelector.x,
+                    y:that.nodeSelector.y
+                } );
+
+                that.nodeSelector.hide();
+
+            }
+        } //]
+
+        // @ char.
+        if( evt.charCode == 64 ) { //[
+
+            if ( that.nodeSelector.isVisible === true ) {
+
+                that.addNode({
+                    type:"Perldata::Array",
+                    x:that.nodeSelector.x,
+                    y:that.nodeSelector.y
+                } );
+
+                that.nodeSelector.hide();
+
+            }
+
+        } //]
+
+    } ) //]
+
+    ; // end $container //]
+
+    // end events //]
+    
 } //]
 
-PNI.Scenario.prototype = {
+PNI.Scenario.prototype = { //[
 
-    // TODO dovrebbe chiamarsi addNodeToModel ??
     addNode: function(requestData) { //[
 
         requestData.scenario_id = this.model.id;
@@ -197,6 +259,12 @@ PNI.Scenario.prototype = {
                 arg.scenario = that;
                 var node;
 
+                if( arg.model.type == "Perldata::Array" ) {
+                    node = new PNI.Node.Perldata.Array(arg);
+                }
+                if( arg.model.type == "Perldata::Hash" ) {
+                    node = new PNI.Node.Perldata.Hash(arg);
+                }
                 if( arg.model.type == "Perldata::Scalar" ) {
                     node = new PNI.Node.Perldata.Scalar(arg);
                 }
@@ -205,63 +273,78 @@ PNI.Scenario.prototype = {
                 }
 
                 that.nodes.push(node);
+
+                // Make new node the currentNode.
+                that.currentNode = node;
             }
-        });
+
+        } );
 
     }, //]
 
-    //getModel: function() { },
+    delNode: function(node) { //[
 
-    runTask: function() {
+        $j.ajax({
+            type: 'DELETE',
+            url: '/node/'+node.model.id,
+            success: function(responseData) {
 
-        var requestData = {};
+                node.destroy();
+
+            }
+        } );
+
+    }, //]
+
+    runTask: function() { //[
 
         var that = this;
-        //TODO uniformala scrivendola come 'GET'
-        // ma bisogna fare una GET ?
-        $j.getJSON( 
-            '/scenario/'+that.model.id+'/task', 
-            requestData, function(responseData) {
+
+        $j.ajax({
+            type: 'GET',
+            url: '/scenario/'+this.model.id+'/task',
+            success: function(responseData) {
                 that.model = responseData;
                 that.renderView();
-            });
+            }
+        } );
 
-    },
+    }, //]
 
     updateView: function() { //[
 
-
-
-        // TODO per ora devo resettare nodes e edges
-        this.nodes=[];
-        this.edges=[];
-
-        var requestData = {};
-
-        var controllerRoute = '/scenario/'+this.model.id;
-
         var that = this;
 
-        /*
-        * TODO uniformala alle altre, dovrebbe essere qualcosa del genere
-        $j.ajax(
-        type: 'GET',
-        url: '/scenario',
-        success: function(responseData) 
-        */
+        $j.ajax({
+            type: 'GET',
+            url: '/scenario/'+this.model.id,
+            success: function(responseData) {
+                that.model = responseData;
+                that.renderView();
+            }
+        } );
 
-        $j.getJSON( controllerRoute, requestData, function(responseData) {
-
-            that.model = responseData;
-            that.renderView();
-        });
     }, //]
 
-    renderView: function() {
+    renderView: function() { //[
+
+        // TODO per ora devo resettare nodes e edges
+        this.layer.clear();
+        for ( var i in this.nodes ) {
+            this.nodes[i].destroy();
+            console.log(this.nodes[i]);
+        }
+        for ( var i in this.edges ) {
+            this.edges[i].destroy();
+        }
+        this.nodes=[];
+        this.edges=[];
 
         for ( var i in this.model.nodes ) {
 
             var model = this.model.nodes[i];
+            var maybeNode = PNI.elemById[model.id];
+            console.log(maybeNode);
             // TODO dovrebbe funzionare prendere il nodo da PNI.elemById
             var node;
 
@@ -269,19 +352,33 @@ PNI.Scenario.prototype = {
             arg.scenario = this;
             arg.model = model;
 
+            if( model.type == "JS::THREE::Scene" ) {
+                // TODO lo lascio qua giusto per fare lo sborone
+                require(['JS/THREE/Scene']);
+            }
+
+            if( model.type == "Perldata::Array" ) {
+                node = new PNI.Node.Perldata.Array(arg);
+            }
+            if( model.type == "Perldata::Hash" ) {
+                node = new PNI.Node.Perldata.Hash(arg);
+            }
             if( model.type == "Perldata::Scalar" ) {
                 node = new PNI.Node.Perldata.Scalar(arg);
             }
             else {
                 node = new PNI.Node(arg);
             }
+
             this.nodes.push(node);
+
         }
 
         for ( var i in this.model.edges ) {
 
             var model = this.model.edges[i];
-            // TOD var edge = PNI.elemById[model.id];
+            var maybeEdge = PNI.elemById[model.id];
+            console.log(maybeEdge);
 
             var arg = {};
             arg.source = PNI.elemById[model.source_id];
@@ -293,10 +390,12 @@ PNI.Scenario.prototype = {
             edge.drawLine();
 
             this.edges.push(edge);
-        } 
-    }
 
-};
+        } 
+
+    } //]
+
+}; //]
 
 PNI.extend(PNI.Scenario,PNI.Elem);
 
@@ -304,7 +403,7 @@ PNI.extend(PNI.Scenario,PNI.Elem);
 
 // PNI.NodeSelector //[
 
-PNI.NodeSelector = function(arg) {
+PNI.NodeSelector = function(arg) { //[
 
     this.x = undefined;
     this.y = undefined;
@@ -319,9 +418,9 @@ PNI.NodeSelector = function(arg) {
 
     this.isVisible = false;
 
-}
+} //]
 
-PNI.NodeSelector.prototype = {
+PNI.NodeSelector.prototype = { //[
 
     hide: function() { 
 
@@ -341,7 +440,7 @@ PNI.NodeSelector.prototype = {
         this.$input.val("");
 
         this.$container
-        .css({left:this.x,top:this.y})
+        .css({left:this.x,top:this.y} )
         .show()
         ;
 
@@ -355,13 +454,13 @@ PNI.NodeSelector.prototype = {
 
         $j.getJSON( 'node_list', function(nodeList) { 
 
-            that.$input.autocomplete({source:nodeList}); 
+            that.$input.autocomplete({source:nodeList} ); 
 
-        });
+        } );
 
     }
 
-}; 
+}; //]
 
 //]
 
@@ -386,7 +485,7 @@ PNI.Node = function(arg) {
     this.ins = [];
     this.outs = [];
 
-    this.group = new Kinetic.Group({draggable:true});
+    this.group = new Kinetic.Group({draggable:true} );
 
     // Create box. //[
 
@@ -400,7 +499,7 @@ PNI.Node = function(arg) {
             stroke: "black",
             strokeWidth: PNI.Theme.strokeWidth,
 
-        });
+        } );
 
         this.group.add(this.box);
 
@@ -420,7 +519,7 @@ PNI.Node = function(arg) {
             align: "center",
             verticalAlign: "top",
 
-        });
+        } );
 
         this.group.add(this.label);
 
@@ -469,10 +568,14 @@ PNI.Node = function(arg) {
     this.layer.add(this.group);
     this.scenario.stage.add(this.layer);
 
+    // events //[
+
     var that = this;
 
-    this.group.on( "mouseover", function(evt) { document.body.style.cursor = "pointer"; });
-    this.group.on( "mouseout", function(evt) { document.body.style.cursor = "default"; });
+    this.group.on( 'click', function(evt) { that.scenario.currentNode = that; } );
+
+    this.group.on( "mouseover", function(evt) { document.body.style.cursor = "pointer"; } );
+    this.group.on( "mouseout", function(evt) { document.body.style.cursor = "default"; } );
     this.group.on( "dragmove", function(evt) {
 
         // Update node position.
@@ -489,21 +592,33 @@ PNI.Node = function(arg) {
 
             }
         }
-    });
+    } );
     this.group.on( "dragend",function(evt) {
 
         var requestData = {x:that.x,y:that.y};
 
         $j.ajax({
-            type: 'PUT',
+            type: 'POST',
             url: '/node/'+that.model.id,
             data: requestData
-        });
+        } );
 
-    });
+    } );
+
+    // end events //]
 }
 
-PNI.Node.prototype = { };
+PNI.Node.prototype = { 
+
+    // TODO forse  meglio rinominarlo in cleanUp ?
+    // QUESTO E' UNO DI QUEI METODI CHE DEVONO RICHIAMARE __SUPER__
+    destroy: function() {
+
+        this.scenario.stage.remove(this.layer);
+
+    }
+
+};
 
 PNI.extend(PNI.Node,PNI.Elem);
 
@@ -511,14 +626,16 @@ PNI.extend(PNI.Node,PNI.Elem);
 
 PNI.Node.Perldata = {};
 
-PNI.Node.Perldata.Scalar = function(arg) {
+// PNI.Node.Perldata.Scalar //[
+
+PNI.Node.Perldata.Scalar = function(arg) { //[
 
     PNI.Node.call( this, arg );
 
     this.$dataEditor = $j('<div class="ui-widget"></div>')
-    .css({top:this.y,left:this.x+this.height})
-    .css({height:this.height,width:10})
-    .css({position:"absolute",display:"block"})
+    .css({top:this.y,left:this.x+this.height} )
+    .css({height:this.height,width:10} )
+    .css({position:"absolute",display:"block"} )
     .appendTo(this.scenario.$container)
     .show()
     ;
@@ -531,10 +648,12 @@ PNI.Node.Perldata.Scalar = function(arg) {
     .focus()
     ;
 
+    // events //[
+
     var that = this;
 
     this.$dataEditorInput
-    .click( function(evt) { that.$dataEditorInput.focus(); } )
+    //.click( function(evt) { that.$dataEditorInput.focus(); } )
     .focusout( function(evt) {
 
         // Update model if input data is changed.
@@ -549,12 +668,41 @@ PNI.Node.Perldata.Scalar = function(arg) {
                 url: '/slot/'+that.model.ins[0].id+'/data',
                 data: requestData,
                 success: function(responseData) {}
-            });
+            } );
 
         } 
-    })
+
+    } )
+    .keypress( function( evt ) { //[
+
+        // ENTER.
+        if( evt.charCode == 13 ) { //[
+
+        //TODO COPIATO MALAMENTE DA QUI SOPRA
+
+        // Update model if input data is changed.
+        if( that.model.ins[0].data != that.$dataEditorInput.val() ) {
+
+            that.model.ins[0].data = that.$dataEditorInput.val();
+            var requestData = { data: that.model.ins[0].data };
+
+            // TODO dovrebbe essere un metodo gestito da PNI.Slot
+            $j.ajax({
+                type: 'POST',
+                url: '/slot/'+that.model.ins[0].id+'/data',
+                data: requestData,
+                success: function(responseData) {}
+            } );
+
+        } 
+
+        } //]
+
+    } ) //]
     ;
 
+    this.group.on( "click", function(evt) { that.$dataEditorInput.toggle(); } );
+ 
     this.group.on( "dragmove", function(evt) {
 
         // TODO questo e' preso dalla classe padre, CERCA DI FARE QUALCOSA PIU FURBO,
@@ -565,7 +713,7 @@ PNI.Node.Perldata.Scalar = function(arg) {
         that.y = position.y;
         that.x = position.x;
 
-        that.$dataEditor.css({top:that.y,left:that.x+that.height});
+        that.$dataEditor.css({top:that.y,left:that.x+that.height} );
 
         // Draw out edges.
         for ( var i in that.outs ) {
@@ -576,32 +724,58 @@ PNI.Node.Perldata.Scalar = function(arg) {
 
             }
         }
+    } );
 
-    });
+    // end events //]
 
-    /*
-    var label;
-    var inId = arg.ins[0];
+} //]
 
-    $j.ajax({
-    type: 'GET',
-    url: '/slot/'+inId+'/data',
-    success: function(responseData) {
+PNI.Node.Perldata.Scalar.prototype = {
+
+    // TODO dovrei richiamare anche il destroy del padre, per ora scrivo il
+    // codice direttamente, cioe la remove del layer.
+    destroy: function() {
+
+        this.scenario.stage.remove(this.layer);
+
+        this.$dataEditor.remove();
 
-    // TODO BRUTTISSIMA SOLUZIONE ma basta che funzioni per ora
-    label = responseData;
-    arg.label = label;
-    PNI.Node.call( this, arg );
     }
-    });
-    */
 
-}
-
-PNI.Node.Perldata.Scalar.prototype = {};
+};
 
 PNI.extend(PNI.Node.Perldata.Scalar,PNI.Node);
 
+//]
+
+// PNI.Node.Perldata.Array //[
+
+PNI.Node.Perldata.Array = function(arg) { //[
+
+    PNI.Node.call( this, arg );
+
+} //]
+
+PNI.Node.Perldata.Array.prototype = {};
+
+PNI.extend(PNI.Node.Perldata.Array,PNI.Node);
+
+//]
+
+// PNI.Node.Perldata.Hash //[
+
+PNI.Node.Perldata.Hash = function(arg) { //[
+
+    PNI.Node.call( this, arg );
+
+} //]
+
+PNI.Node.Perldata.Hash.prototype = {};
+
+PNI.extend(PNI.Node.Perldata.Hash,PNI.Node);
+
+//]
+
 // PNI.Slot //[
 
 PNI.Slot = function(arg) {
@@ -622,21 +796,21 @@ PNI.Slot = function(arg) {
         stroke: "black",
         strokeWidth: PNI.Theme.strokeWidth,
 
-    });
+    } );
 
     var that = this;
 
     this.box.on("mouseover", function(evt) {
 
-        that.scenario.currenSlot = that;
+        that.scenario.currentSlot = that;
 
-    });
+    } );
 
     this.box.on("mouseout", function(evt) {
 
-        that.scenario.currenSlot = undefined;
+        that.scenario.currentSlot = undefined;
 
-    });
+    } );
 
 }
 
@@ -654,7 +828,7 @@ PNI.Slot.prototype = {
                 that.data = responseData;
 
             }
-        });
+        } );
 
         return this.data;
     },
@@ -679,7 +853,7 @@ PNI.extend(PNI.Slot,PNI.Elem);
 
 // PNI.In //[
 
-PNI.In = function(arg) {
+PNI.In = function(arg) { //[
 
     PNI.Slot.call( this, arg );
 
@@ -701,7 +875,6 @@ PNI.In = function(arg) {
             // that.scenario.stage.off( "mousemove" );
             that.scenario.stage.on( "mousemove" , function() {} );
 
-
             var requestData = {
                 scenario_id: edge.scenario.model.id,
                 source_id: edge.source.model.id,
@@ -716,17 +889,20 @@ PNI.In = function(arg) {
                     edge.model = responseData; 
                     PNI.elemById[edge.model.id] = edge;
                 }
-            });
+
+            } );
+
         }
-    });
 
-}
+    } );
 
-PNI.In.prototype = { 
+} //]
+
+PNI.In.prototype = { //[
 
     getY: function() { return this.node.y - PNI.Theme.slotSide; }
 
-};
+}; //]
 
 PNI.extend(PNI.In,PNI.Slot);
 
@@ -734,7 +910,7 @@ PNI.extend(PNI.In,PNI.Slot);
 
 // PNI.Out //[
 
-PNI.Out = function(arg) {
+PNI.Out = function(arg) { //[
 
     PNI.Slot.call( this, arg );
 
@@ -751,20 +927,95 @@ PNI.Out = function(arg) {
         edge.drawLine();
         that.edges.push(edge);
 
-    });
+    } );
 
-}
+} //]
 
-PNI.Out.prototype = {
+PNI.Out.prototype = { //[
 
     getY: function() { return this.node.y + this.node.height; }
 
-};
+}; //]
 
 PNI.extend( PNI.Out, PNI.Slot );
 
 //]
 
+// PNI.Toolbar //[
+
+PNI.Toolbar = function(arg) { //[
+
+    this.scenario = arg.scenario;
+    this.x = 10;
+    this.y = 10;
+
+    this.$container = $j('<div class="ui-widget"></div>')
+    .css({top:this.y,left:this.x,"z-index":1} )
+    .css({position:"absolute",display:"block"} )
+    .appendTo(this.scenario.$container)
+    .show()
+    ;
+
+    var that = this;
+
+    this.$taskButton = $j('<button id="task">task</button>')
+    .click( function() {
+        that.scenario.runTask();
+        that.scenario.renderView();
+    } )
+    .button()
+    .appendTo(this.$container)
+    .show()
+    ;
+
+    // TODO vedi bene la toolbar di jQuery.
+    this.$delNodeButton = $j('<button id="del">del</button>')
+    .click( function() {
+
+        var node = that.scenario.currentNode;
+
+        if ( node != "undefined" ) {
+            that.scenario.delNode(node);
+            // TODO vedi se la renderView deve stare nel metodo delNode,
+            // vedi anceh per gli altri casi
+            that.scenario.renderView();
+        }
+
+    } )
+    .button()
+    .appendTo(this.$container)
+    .show()
+    ;
+
+} //]
+
+PNI.Toolbar.prototype = { };
+
+//]
+
+// PNI.SemiEdge //[
+
+PNI.SemiEdge = function(arg) { //[
+
+    this.startX;
+    this.startY;
+    this.endX;
+    this.endY;
+
+    // TODO questa classe rappresenta un edge ancora non collegato
+    // si deve vedere come una linea, poi l' edge collegato invece e' come una
+    // bezier. Sull' onrelease del mouse non si deve distruggere perch ogni
+    // scenario ne ha un' istanza, deve pero sparire.
+} //]
+
+PNI.SemiEdge.prototype = { //[
+
+    drawLine: function() { }
+
+}; //]
+
+//]
+
 // PNI.Edge //[
 
 PNI.Edge = function(arg) { //[
@@ -782,7 +1033,7 @@ PNI.Edge = function(arg) { //[
 
         this.scenario.stage.on( "mousemove", function(evt) {
             that.drawLine();
-        });
+        } );
 
         this.scenario.semiEdge = this;
 
@@ -793,6 +1044,12 @@ PNI.Edge = function(arg) { //[
 
 PNI.Edge.prototype = {
 
+    destroy: function() {
+
+        this.scenario.stage.remove(this.layer);
+
+    },
+
     drawLine: function() { //[
 
         var context = this.layer.getContext();
