@@ -3,9 +3,21 @@ use PNI::Mo;
 extends 'PNI::Slot';
 
 use PNI::Edge;
+use PNI::Elem;
 
 has edge => ();
 
+sub by_id {
+    my $elem = PNI::Elem::by_id(@_);
+
+    if ( defined $elem and $elem->isa('PNI::In') ) {
+        return $elem;
+    }
+    else {
+        return;
+    }
+}
+
 sub is_connected { return defined( shift->edge ) ? 1 : 0 }
 
 1;
@@ -35,6 +47,14 @@ hold only one edge.
 
 =head1 METHODS
 
+=head2 by_id
+
+    use PNI::In;
+
+    my $in = PNI::In::by_id($in_id);
+
+Given a slot id, returns a reference to the slot.
+
 =head2 is_connected
 
     $in->is_connected;
