@@ -16,7 +16,7 @@ function init() {
     renderer = new THREE.CanvasRenderer();
     renderer.setSize( window.innerWidth, window.innerHeight );
 
-    var d = document.getElementById('dashboard-tab');
+    var d = document.getElementById('scenario');
     d.appendChild( renderer.domElement );
 
 }
@@ -38,8 +38,7 @@ function render() {
 
 }
 
-var camera, scene, renderer,
-geometry, material, mesh;
+var camera, scene, renderer, geometry, material, mesh;
 
 init();
 animate();
