@@ -3,10 +3,9 @@ package    # Avoid PAUSE indexing.
 use PNI::Node::Mo;
 extends 'PNI::Node';
 
-sub BUILD {
-    my $self = shift;
-    $self->in('content');
-}
+sub BUILD { shift->label('#'); }
+
+sub task { }
 
 1;
 
