@@ -2,7 +2,11 @@ package PNI::Node::Empty;
 use PNI::Mo;
 extends 'PNI::Node';
 
-sub BUILD { }
+sub BUILD {
+    my $self = shift;
+    $self->in;
+    $self->out;
+}
 
 sub task { }
 
